﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.EvolutionTracking {
  [StorableClass]
  [Item("AfterCrossoverOperator", "A generic operator that can record genealogical relationships between crossover parents and children.")]
  public class AfterCrossoverOperator<T> : EvolutionTrackingOperator, ICrossoverOperator<T> where T : class,IItem {
    private const string defaultParentsParameterName = "Parents";
    private const string defaultChildParameterName = "Child";
    public IScopeTreeLookupParameter<T> ParentsParameter { get; set; }
    public ILookupParameter<T> ChildParameter { get; set; }

    protected AfterCrossoverOperator(AfterCrossoverOperator<T> original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new AfterCrossoverOperator<T>(this, cloner);
    }

    public AfterCrossoverOperator() {
      ParentsParameter = new ScopeTreeLookupParameter<T>(defaultParentsParameterName);
      ChildParameter = new LookupParameter<T>(defaultChildParameterName);
      Parameters.Add(ParentsParameter);
      Parameters.Add(ChildParameter);
    }

    public override IOperation Apply() {
      return base.Apply();
    }
  }
}
